###########################################################################
## LAB: LINEAR REGRESSION
##
## The br.csv data are taken from Principles of Econometrics, by 
## Hill, Griffihs and Lim (Wiley, 2008). 
## 1080 observations on the prices and characteristics of houses sold 
## in Baton Rouge, LA 2005. The variables considered are:
## price sqft Bedrooms Baths Age Occupancy Pool Style Fireplace Waterfront DOM
## price, dollars 
## sqft		total square feet
## bedrooms	number of bedrooms
## baths		number of full baths
## age		age in years
## Occupancy	Owner = 1, Vacant = 2, Tenant = 3
## Pool		Yes = 1, No = 0   
## Style		Traditional = 1, Townhouse = 2, Ranch = 3, New Orleans = 4,
##          Mobile Home = 5, Garden = 6, French = 7, Cottage = 8, 
##          Contemporary = 9, Colonial = 10, Acadian = 11
## Fireplace	Yes = 1, No = 0	
## Waterfront	Yes =1, No = 0
## DOM		Days on the market
###########################################################################


## 0. ###########################
## Loading data 
# set directory in the folder containing the data following:
# Session -> Set Working Directory -> Choose Directory

br = read.table("br.csv", sep = ",", header = T) # reads the data from a csv file
br = read.csv("br.csv") # reads the data from a csv file
attach(br)

## 1. ###########################
## Descriptive statistics 

vCategorical = c("Occupancy", "Style")
vIndexCategorical = !(names(br) %in% vCategorical)
tBR = br[ , vIndexCategorical]
mBR = as.matrix(tBR)

# Variance-covariance matrix
vMeanBR = apply(X = mBR, MARGIN = 2, FUN = mean)
n = nrow(mBR)
mVarcov = 1/n * (t(mBR) %*% mBR) - (vMeanBR %*% t(vMeanBR)) # one way of calculating the covariance matrix
mVarcov = cov(mBR) # another way of calculating the covariance matrix

# Distribution of prices
plot(price)
hist(x = price, breaks = 100) # Histogram
hist(x = price, breaks = 100, probability = TRUE)   # Rescaled histogram
lines(density(price), col = 'blue', lwd = 2)
hist(log(price), 30, col = "red")  # Histogram of log transformation 
summary(price)

# Relationship between price and other variables
plot(sqft, price, col = "red") # Scatterplot of prices vs sqft
plot(Age, price, col = "red") # Scatterplot of prices vs age (note: age is discrete)
plot(factor(Occupancy), price, col ='red') # Boxplots by Occupancy
plot(factor(Occupancy), price, col ='red', outline = FALSE) # Boxplots without otuliers
plot(factor(Style), price, col = 'yellow') 
pairs(price ~ sqft + Age, upper.panel = panel.smooth)


## 2. ###########################
## Regression model

# Simple linear regression
regr0 = lm(price ~ sqft)
summary(regr0)
plot(x = sqft, y = price, col = "blue")
lines(x = sqft, y = regr0$fitted.values, col = "red")

# Multiple linear regression
regr = lm(price ~ sqft + Age + Pool + Bedrooms + Fireplace + Waterfront + DOM) 
summary(regr)  # This summary provides all the relevant info on the model fit 

## categorical variables in regression
regr2 = lm(price ~ 
             sqft + Age + Pool + Bedrooms + Pool + Fireplace +
             Waterfront + DOM + factor(Occupancy) + factor(Style)) 
summary(regr2)

## adding and removing variables 
summary(update(regr2,  .~. - Fireplace))
summary(update(regr2,  .~. - factor(Occupancy)))

## Handling the linear regression object "regr": the functions that follow
## allow to carry out our analyses using the linear model previously fitted

N = nrow(br) # number of observations
# the number of variables (p) equals the number of non-categorical
# (discrete or continuous) variables plus the number of levels of each
# categorical variable minus 1 (for each variable), which is the baseline 
# factor level captured by the intercept.
p = 7 + length(levels(Style)) - 1

df = N - p - 1 # degrees of freedom
df.residual(regr) # degrees of freedom using the formula
yf = fitted(regr)   # fitted values
e = residuals(regr) # residuals of the regression
vCoeffs = coef(regr) # coefficients of the regression
mCovBetas = vcov(regr) # variance-covariance matrix of the coeffs of the regression
sqrt(CovBeta[2,2]) # std err of beta_1_hat (compare with table)
vCoeffs[2]/sqrt(CovBeta[2,2]) # t statistic - same as obtained in summary(regr)
confint(regr, level=0.95) # Confidence intervals for model parameters 
TSS = sum((price - mean(price))^2) # total sum of squares
RSS = sum(e^2) # residuals sum of squares
ESS = TSS - RSS # explained sum of squares

SER = sqrt(RSS / df) # rounding difference with respect to summary(regr)
R2 = (TSS - RSS)/TSS
AdjR2 = 1 - (RSS/df)/(TSS/(N - 1))
Fstat = (ESS/p) / (RSS/df)
pvalF = 2 * pf(abs(Fstat), df1 = p, df2 = df, lower.tail = FALSE) 
tstat = vCoeffs[3] / sqrt(mCovBetas[3, 3]) # Age
pvalT = 2 * pt(abs(tstat), df = df, lower.tail = FALSE)
# The p-value (pvalF and pvalT) corresponds to the probability of observing
# by pure chance a sample as adverse to the null hypothesis (H0) as ours, while H0
# being true. Low p-values provide evidence against H0, which is to be rejected. 
# The opposite is true for high p-values (failing to reject H0).

## 3. ################################################
## Handling outliers and high leverage points

h = hatvalues(regr) # leverage of the observations (diagonal elements of the hat matrix H)
r = rstandard(regr) # formula at slide 1.20
plot(h, r) # it helps us to detect points with high leverange and high residuals (outliers)
text(h, r, labels = names(h))
bad = h > quantile(x = h, probs = 0.9) & 
  r > quantile(x = r, probs = 0.9)

brClean = br[!bad,] # excluding row indices corresponding to bad observations
regr2 = lm(price ~ sqft + Age + Pool + Bedrooms +
             Fireplace + Waterfront + DOM + Style, data = brClean) 
summary(regr2)
h2 = hatvalues(regr2)
r2 = rstandard(regr2)

colors = rep("black", nrow(br))
colors[bad] = "red"
par(mfrow = c(2,1))
plot(h, r, main = "All observations", xlim = c(0, 0.14), ylim = c(-10, 12),
     col = colors)
plot(h2, r2, main = "After removing critical observations", xlim = c(0, 0.14), ylim = c(-10, 12))
par(mfrow = c(1,1))

## 4. ################################################
## Residuals' analysis

# Underlying our regression model, it was assumed that residuals are normally
# distributed with constant variance. Failure of these hypothesis to hold
# imply that our test of hypothesi would not work well.
# Let us verify such hypothese graphically. 

# Checking normality
hist(e, 60, col = 'blue', main = ' ', probability = TRUE)
curve(dnorm(x, mean = mean(e), sd = sd(e)), 
      col = "red", lwd = 2, add = TRUE) 

hist(r, 60, col = 'blue', main = ' ', probability = TRUE) 
curve(dnorm(x, mean = 0, sd = 1), 
      col = "red", lwd = 2, add = TRUE) 
# Checking constant variance (homoskedastcity)

plot(price, e, col = "red")  # residuals vs observed values
abline(h = 0) 
# variability in the residuals is not constant but it does 
# not show a marked relationship with prices
